%%
% Implements the minimization of the generalized BB energy using PD.
%     min_{U} J_epsilon(K(U) + b) + i_I(U)
%  min_x F(K(U)) + G(U)   where   F(V)=J_epsilon(V + b)


%% Copyright (c) 2013 Gabriel Peyre, Nicolas papadakis, Edouard Oudet
close all
addpath('toolbox/');


 P = 32; Q = 32;

d = [P,Q];
epsilon = 1e-8;


%exposant of the generalized cost
alpha= 1; % should be in [0;1];


mynorm = @(a)norm(a(:));
dotp = @(a,b)sum(a(:).*b(:));
sum2 = @(a)sum(a(:));
X = 0:1./(P-1):1;
gaussian = @(a,sigma)exp( -((X-a).^2)/(2*sigma^2) );
normalize = @(u)u/sum(u(:));


%To represent the weights
weight=ones(P,Q);

%%
% Load the data.
test = 'gaussian';
%test = 'obsession';
%test = 'mixture';

%test='weight';

sigma =  .1;
rho = 1e-12; % minimum density value
switch test
    
    case 'gaussian'
        f0 = (rho + gaussian(.2,sigma/2.)/4.)';
        f1 = (rho + gaussian(.6,sigma))';
        epsilon=min(f0(:));
        
        
        
    otherwise
        error('Unknown');
end


masse0=sum(f0(:));
masse1=sum(f1(:));

facteur=log(masse1/masse0);


clf; 

maxi=max(max(f0(:)),max(f1(:)));

     subplot(2,1,1);
       plot(f0');
       axis([1 P 0 maxi])
       set(gca,'ytick',[])
       set(gca,'xtick',[])
       
      subplot(2,1,2);
       plot(f1');
       axis([1 P 0 maxi])
       set(gca,'ytick',[])
       set(gca,'xtick',[])  
       
       drawnow;

%%
% Initialization using linear interpolation of the densities.

t = repmat( 0:1./Q:1, [ P 1]);

f_init = (1-t) .* repmat(f0, [1  Q+1]) + t .* repmat(f1, [1 Q+1]);

% linear operator and adjoint
add_constraints = 1;

K  = @(X)pd_operator(X, +1);
KS = @(X)pd_operator(X, -1);
L = 1;
if 0
    % test for operator norm
    u = interp_adj(randn(N,P,Q,3));
    u = u*(1/norm(u)); e = [];
    for i=1:30
        v = KS(K(u));
        e(end+1) = dotp_stag(u,v);
        u = v * (1/norm(v));
    end
    L = e(end);
end

% to add into the J functional, J(K(U)+b) 
b = zeros(P,Q,2);
b(:,1,2) = f0/2; b(:,end,2) = f1/2;

% test for adjointness
U1 = interp_adj(randn(P,Q,2));
U2 = interp_adj(randn(P,Q,2));
if abs(dotp( K(U1), K(U2) ) - dotp_stag(KS(K(U1)), U2))>1e-5
    warning('Adjointness problem');
end

% prox operators
proxJeps  = @(V,gamma)proxJ(V,gamma,epsilon,alpha,weight);
proxF    = @(X,gamma)proxJeps(X+b,gamma)-b;
proxFS   = compute_dual_prox(proxF);
proxG    = @(U,gamma)pd_div_proj(U,facteur);
% functionals
J = @(V)sum2(  sum(V(:,:,1).^2,4) ./ (max((V(:,:,2)),max(epsilon,1e-10)).^alpha)  );  %pb interpU not >0  



%%
% Run the algorithm.

% initialization
U0 = staggered(d); U0.M{2} = f_init; 
% parameters
options.theta = 1.;
options.sigma=85;
options.tau = .99/(options.sigma*L);
options.niter = 5000;
mymin = @(x)min(min(min(x(:,:,2))));

options.report = @(U,V)struct( 'J', J(interp(U)), 'Constr', mynorm(div(U)), 'Min', mymin(interp(U)));%,...%


tic
[U,R,V] = perform_primal_dual(U0, K,  KS, proxFS, proxG, options);
toc
Jlist  = s2v(R,'J');
Constr = s2v(R,'Constr');
MinVal = s2v(R,'Min');


%%
% Display the resulting density \(f(x,t)\) for \(t\) from 0 to 1.

sel = round(linspace(1,Q+1,20));
V   = interp(U);

maxi=max(U.M{2}(:));
 figure;
 cpt=1;
 for i=1:4
     for j=1:5,
       subplot(5,4,cpt);
       plot(U.M{2}(:,sel(cpt))');
       axis([1 P 0 maxi])
       set(gca,'ytick',[])
       set(gca,'xtick',[])
       cpt=cpt+1;
     end
 end
 

 figure;
subplot(2,1,1);
plot(Jlist(10:end), '-'); axis tight;
title('J');
subplot(2,1,2);
plot((Constr(10:end)), '.-'); axis tight;
title('div=0 violation');




